@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -61,7 +61,7 @@ RunControl *AndroidAnalyzeSupport::createAnalyzeRunControl(AndroidRunConfigurati
     params.sysroot = SysRootKitInformation::sysRoot(target->kit()).toString();
     // TODO: Not sure if these are the right paths.
     params.workingDirectory = target->project()->projectDirectory().toString();
-    if (runMode == ProjectExplorer::QmlProfilerRunMode) {
+    if (runMode == QmlProfilerRunMode) {
         QTcpServer server;
         QTC_ASSERT(server.listen(QHostAddress::LocalHost)
                    || server.listen(QHostAddress::LocalHostIPv6), return 0);
@@ -76,64 +76,48 @@ RunControl *AndroidAnalyzeSupport::createAnalyzeRunControl(AndroidRunConfigurati
 
 AndroidAnalyzeSupport::AndroidAnalyzeSupport(AndroidRunConfiguration *runConfig,
     AnalyzerRunControl *runControl)
-    : AndroidRunSupport(runConfig, runControl),
-      m_runControl(0),
+    : QObject(runControl),
       m_qmlPort(0)
 {
-    if (runControl) {
-        m_runControl = runControl;
-        connect(m_runControl, SIGNAL(starting(const Analyzer::AnalyzerRunControl*)),
-                m_runner, SLOT(start()));
-    }
-    connect(&m_outputParser, SIGNAL(waitingForConnectionOnPort(quint16)),
-            SLOT(remoteIsRunning()));
-    connect(m_runner, SIGNAL(remoteProcessStarted(int)),
-            SLOT(handleRemoteProcessStarted(int)));
-    connect(m_runner, SIGNAL(remoteProcessFinished(QString)),
-            SLOT(handleRemoteProcessFinished(QString)));
+    QTC_ASSERT(runControl, return);
 
-    connect(m_runner, SIGNAL(remoteErrorOutput(QByteArray)),
-            SLOT(handleRemoteErrorOutput(QByteArray)));
-    connect(m_runner, SIGNAL(remoteOutput(QByteArray)),
-            SLOT(handleRemoteOutput(QByteArray)));
-}
+    auto runner = new AndroidRunner(this, runConfig, runControl->runMode());
 
-void AndroidAnalyzeSupport::handleRemoteProcessStarted(int qmlPort)
-{
-    m_qmlPort = qmlPort;
-}
+    connect(runControl, &AnalyzerRunControl::finished,
+        [runner]() { runner->stop(); });
 
-void AndroidAnalyzeSupport::handleRemoteProcessFinished(const QString &errorMsg)
-{
-    if (m_runControl)
-        m_runControl->notifyRemoteFinished();
-    AndroidRunSupport::handleRemoteProcessFinished(errorMsg);
-}
+    connect(runControl, &AnalyzerRunControl::starting,
+        [runner]() { runner->start(); });
 
-void AndroidAnalyzeSupport::handleRemoteOutput(const QByteArray &output)
-{
-    const QString msg = QString::fromUtf8(output);
-    if (m_runControl)
-        m_runControl->logApplicationMessage(msg, Utils::StdOutFormatSameLine);
-    else
-        AndroidRunSupport::handleRemoteOutput(output);
-    m_outputParser.processOutput(msg);
-}
+    connect(&m_outputParser, &QmlDebug::QmlOutputParser::waitingForConnectionOnPort,
+        [this, runControl](quint16) {
+            runControl->notifyRemoteSetupDone(m_qmlPort);
+        });
 
-void AndroidAnalyzeSupport::handleRemoteErrorOutput(const QByteArray &output)
-{
-    const QString msg = QString::fromUtf8(output);
-    if (m_runControl)
-        m_runControl->logApplicationMessage(msg, Utils::StdErrFormatSameLine);
-    else
-        AndroidRunSupport::handleRemoteErrorOutput(output);
-    m_outputParser.processOutput(msg);
-}
+    connect(runner, &AndroidRunner::remoteProcessStarted,
+        [this](int, int qmlPort) {
+            m_qmlPort = qmlPort;
+        });
 
-void AndroidAnalyzeSupport::remoteIsRunning()
-{
-    if (m_runControl)
-        m_runControl->notifyRemoteSetupDone(m_qmlPort);
+    connect(runner, &AndroidRunner::remoteProcessFinished,
+        [this, runControl](const QString &errorMsg)  {
+            runControl->notifyRemoteFinished();
+            runControl->appendMessage(errorMsg, Utils::NormalMessageFormat);
+        });
+
+    connect(runner, &AndroidRunner::remoteErrorOutput,
+        [this, runControl](const QByteArray &output) {
+            const QString msg = QString::fromUtf8(output);
+            runControl->logApplicationMessage(msg, Utils::StdErrFormatSameLine);
+            m_outputParser.processOutput(msg);
+        });
+
+    connect(runner, &AndroidRunner::remoteOutput,
+        [this, runControl](const QByteArray &output) {
+            const QString msg = QString::fromUtf8(output);
+            runControl->logApplicationMessage(msg, Utils::StdOutFormatSameLine);
+            m_outputParser.processOutput(msg);
+        });
 }
 
 } // namespace Internal
